local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local ActionBarsModule = idTweaks:NewModule('ActionBars', 'AceConsole-3.0', 'AceEvent-3.0')
local LibActionButton = LibStub('LibActionButton-1.0')
idTweaks.ActionBars = ActionBarsModule

local TL, TC, TR = 'TOPLEFT', 'TOP', 'TOPRIGHT'
local ML, MC, MR = 'LEFT', 'CENTER', 'RIGHT'
local BL, BC, BR = 'BOTTOMLEFT', 'BOTTOM', 'BOTTOMRIGHT'

-- TODO: Check all classes and all specs, all stances, forms and abilities to see if it works correctly (in other words
-- I didn't and yeah it probably doesn't).
-- TODO: Pet bar.
-- TODO: Vehicle Mode.
-- TODO: "Special" Quest Action Button

function ActionBarsModule:setDefaults()
  --ActionBarsModule:Print('ActionBarsModule:setDefaults')

  self.defaults = {
    profile = {
      enabled = true,
      size = {
        barPadding = 3,
        barMargin = 3,
        buttonPadding = 3,
      },
      bars = {
        stance = {
          enabled = false,
        }
      },
    }
  }
end

function ActionBarsModule:setDatabase()
  --ActionBarsModule:Print('ActionBarsModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('ActionBars', self.defaults)
end

function ActionBarsModule:setOptions()
  --ActionBarsModule:Print('ActionBarsModule:setOptions')

  self.options = {
    name = 'ActionBars',
    handler = self,
    type = 'group',
    args = {
      enabled = {
        order = 1,
        name = 'Enabled',
        desc = 'Shows all bars on top of the basic one.',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.enabled
        end,
        set = function(_, value)
          self.db.profile.enabled = value
        end,
      },

      bars = {
        order = 2,
        name = 'Bars',
        desc = 'Specific Bar Options',
        handler = self,
        type = 'group',
        args = {
          size = {
            order = 1,
            name = 'Size',
            desc = 'Control the size of the frame.',
            type = 'group',
            handler = self,
            args = {
              barMargin = {
                order = 1,
                name = 'Bar Margin',
                type = 'range',
                min = 0,
                max = 50,
                step = 1,
                bigStep = 1,
                get = function()
                  return self.db.profile.size.barMargin
                end,
                set = function(_, value)
                  self.db.profile.size.barMargin = value
                  self.size:sizeAllBars()
                  self.positioning:positionAllBars()
                end
              },

              barPadding = {
                order = 2,
                name = 'Bar Padding',
                type = 'range',
                min = 0,
                max = 50,
                step = 1,
                bigStep = 1,
                get = function()
                  return self.db.profile.size.barPadding
                end,
                set = function(_, value)
                  self.db.profile.size.barPadding = value
                  self.size:sizeAllBars()
                  self.positioning:positionAllBars()
                end
              },

              buttonPadding = {
                order = 3,
                name = 'Button Padding',
                type = 'range',
                min = 0,
                max = 50,
                step = 1,
                bigStep = 1,
                get = function()
                  return self.db.profile.size.buttonPadding
                end,
                set = function(_, value)
                  self.db.profile.size.buttonPadding = value
                  self.size:sizeAllBars()
                  self.positioning:positionAllBars()
                end
              },
            },
          },

          stance = {
            order = 2,
            name = 'Enabled',
            desc = 'Enable the stance bar.',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.bars.stance.enabled
            end,
            set = function(_, value)
              self.db.profile.bars.stance.enabled = value
            end,
          }
        },
      }
    },
  }
end

function ActionBarsModule:registerOptions()
  --ActionBarsModule:Print('ActionBarsModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('ActionBars', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('ActionBars', 'ActionBars', 'idTweaks')
end

function ActionBarsModule:OnInitialize()
  --ActionBarsModule:Print('ActionBarsModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function ActionBarsModule:OnEnable()
  ActionBarsModule:Print('ActionBarsModule:OnEnable')

  self:registerOptions()

  if not self.db.profile.enabled then
    return
  end

  ActionBarsModule:Print('ActionBarsModule:OnEnable ENABLED')

--  self.frames.bar1 = self:createBar1( 1, 12)
--  self.frames.bar2 = self:createBar2(61, 72)
--  self.frames.bar3 = self:createBar3(49, 60)
--  self.frames.bar4 = self:createBar4(25, 36)
--  self.frames.bar5 = self:createBar5(37, 48)

--  self.size:sizeAllBars()
--  self.positioning:positionAllBars()
end

function ActionBarsModule:OnDisable()
  --ActionBarsModule:Print('ActionBarsModule:OnDisable')

  if self.db.profile.enabled then
    -- disable it

    -- bars
    --   stance
    if self.db.profile.bars.stance.enabled then
      -- disable it
    end
  end
end

function ActionBarsModule:createButton(i, buttonName, frameHeader)
  local button = LibActionButton:CreateButton(i, buttonName, frameHeader, {
    outOfRangeColoring = 'button',
    tooltip = 'enabled',
    showGrid = false,
    colors = {
      range = { 0.8, 0.1, 0.1 },
      mana = { 0.5, 0.5, 1.0 }
    },
    hideElements = {
      macro = false,
      hotkey = false,
      equipped = false,
    },
    keyBoundTarget = false,
    clickOnDown = false,
    flyoutDirection = 'UP',
  })

  local icon = _G[buttonName .. 'Icon']
  local texture = (_G[buttonName .. 'NormalTexture2'] or _G[buttonName .. 'NormalTexture'])
  local hotkey = _G[buttonName .. 'HotKey']
  local macrotext = _G[buttonName .. 'Name']

  icon:SetTexCoord(0.08,0.92,0.08,0.92)

  hotkey:Hide()
  macrotext:Hide()

  texture:SetAlpha(0.5)
  texture:SetTexCoord(0,0,0,0)

  -- ugly but works
  texture.SetAlpha = nothing
  hotkey.Show = nothing
  macrotext.Show = nothing

  return button
end

function ActionBarsModule:createBar(barName, buttonName, firstId, lastId, frameHeader)
  ActionBarsModule:Print('ActionBarsModule:createBar', barName, buttonName, firstId, lastId)

  local frame = CreateFrame('Frame', barName, UIParent)

  frame.header = frameHeader
  frame.barName = barName
  frame.buttonName = buttonName
  frame.firstId = firstId
  frame.lastId = lastId

  self.size:sizeBar(frame)

  frame.background = frame:CreateTexture(barName .. 'Background', 'BACKGROUND')
  frame.background:SetAllPoints(frame)
  frame.background:SetColorTexture(0, 0, 0, 0.5)

  return frame
end

function ActionBarsModule:setBarActions(bar, type, firstId, lastId)
  for i = 1, lastId - firstId + 1 do
    bar['button'..i]:SetState(0, type, firstId - 1 + i)
  end
end

ActionBarsModule.size = {}

function ActionBarsModule.size:sizeBar(bar)
--  ActionBarsModule:Print('sizeBar', bar.barName)

  local barPadding = ActionBarsModule.db.profile.size.barPadding
  local buttonPadding = ActionBarsModule.db.profile.size.buttonPadding

  local j = 1
  for i = bar.firstId, bar.lastId do
    local button = ActionBarsModule:createButton(i, bar.buttonName .. i, bar.header)

    if j == 1 then
      ActionBarsModule:Print('sizeBar j is 1', bar.barName, barPadding)

      button:SetPoint(ML, bar, ML, barPadding, 0)
    else
      button:SetPoint(ML, bar['button' .. j - 1], MR, buttonPadding, 0)
    end

    bar['button' .. j] = button

    j = j + 1
  end

  local buttonAmount = bar.lastId + 1 - bar.firstId
  local buttonWidth = bar.button1:GetWidth()
  local buttonHeight = bar.button1:GetHeight()

  local width = buttonWidth * buttonAmount + buttonPadding * (buttonAmount - 1) + barPadding * 2
  local height = buttonHeight + barPadding * 2

  bar:SetWidth(width)
  bar:SetHeight(height)
end

function ActionBarsModule.size:sizeAllBars()
  for b = 1, 5 do
    local bar = ActionBarsModule.frames['bar'..b]

    self:sizeBar(bar)
  end
end

ActionBarsModule.positioning = {}

function ActionBarsModule.positioning:positionAllBars()
  local barMargin = ActionBarsModule.db.profile.size.barMargin

  ActionBarsModule.frames.bar1:SetPoint(BC, UIParent, BC, 0, barMargin)
  ActionBarsModule.frames.bar2:SetPoint(BC, ActionBarsModule.frames.bar1, TC, 0, barMargin)
  ActionBarsModule.frames.bar3:SetPoint(BC, ActionBarsModule.frames.bar2, TC, 0, barMargin)
  ActionBarsModule.frames.bar4:SetPoint(BC, ActionBarsModule.frames.bar3, TC, 0, barMargin)
  ActionBarsModule.frames.bar5:SetPoint(BC, ActionBarsModule.frames.bar4, TC, 0, barMargin)

  MainMenuBar:Hide()
  MultiBarLeft:Hide()
  MultiBarRight:Hide()
end

ActionBarsModule.frames = {}

function ActionBarsModule:createBar1(firstId, lastId)
  local header = CreateFrame('Frame', 'idTweaksActionBar1Header', UIParent, 'SecureHandlerStateTemplate')
  local bar = self:createBar('idTweaksActionBar1', 'idTweaksActionBarButton', firstId, lastId, header)

  self:setBarActions(bar, 'action', firstId, lastId)

  RegisterStateDriver(header, 'page', '0')
  header:SetAttribute('_onstate-page', [[
    self:SetAttribute('state', newstate)
    control:ChildUpdate('state', newstate)
  ]])

  return bar
end

function ActionBarsModule:createBar2(firstId, lastId)
  local header = CreateFrame('Frame', 'idTweaksActionBar2Header', UIParent, 'SecureHandlerStateTemplate')
  local bar = self:createBar('idTweaksActionBar2', 'idTweaksActionBarButton', firstId, lastId, header)

  self:setBarActions(bar, 'action', firstId, lastId)

  RegisterStateDriver(header, 'page', '0')
  header:SetAttribute('_onstate-page', [[
    self:SetAttribute('state', newstate)
    control:ChildUpdate('state', newstate)
  ]])

  return bar
end

function ActionBarsModule:createBar3(firstId, lastId)
  local header = CreateFrame('Frame', 'idTweaksActionBar3Header', UIParent, 'SecureHandlerStateTemplate')
  local bar = self:createBar('idTweaksActionBar3', 'idTweaksActionBarButton', firstId, lastId, header)

  self:setBarActions(bar, 'action', firstId, lastId)

  RegisterStateDriver(header, 'page', '0')
  header:SetAttribute('_onstate-page', [[
    self:SetAttribute('state', newstate)
    control:ChildUpdate('state', newstate)
  ]])

  return bar
end

function ActionBarsModule:createBar4(firstId, lastId)
  local header = CreateFrame('Frame', 'idTweaksActionBar4Header', UIParent, 'SecureHandlerStateTemplate')
  local bar = self:createBar('idTweaksActionBar4', 'idTweaksActionBarButton', firstId, lastId, header)

  self:setBarActions(bar, 'action', firstId, lastId)

  RegisterStateDriver(header, 'page', '0')
  header:SetAttribute('_onstate-page', [[
    self:SetAttribute('state', newstate)
    control:ChildUpdate('state', newstate)
  ]])

  return bar
end

function ActionBarsModule:createBar5(firstId, lastId)
  local header = CreateFrame('Frame', 'idTweaksActionBar5Header', UIParent, 'SecureHandlerStateTemplate')
  local bar = self:createBar('idTweaksActionBar5', 'idTweaksActionBarButton', firstId, lastId, header)

  self:setBarActions(bar, 'action', firstId, lastId)

  RegisterStateDriver(header, 'page', '0')
  header:SetAttribute('_onstate-page', [[
    self:SetAttribute('state', newstate)
    control:ChildUpdate('state', newstate)
  ]])

  return bar
end
