--[[
- Make a category for Elements
  - Make an option for each Element to show/hide it
  * Buffs/Debuffs
  - UnitFrames
    - Each one
  * Castbar / MirrorBar
]]


local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')

local BlizzardUIModule = idTweaks:NewModule('BlizzardUI', 'AceConsole-3.0', 'AceHook-3.0')

function BlizzardUIModule:setDefaults()
  --BlizzardUIModule:Print('BlizzardUIModule:setDefaults')

  self.defaults = {
    profile = {
      buffFrame = {
        hide = true,
      },
      castingBarFrame = {
        hide = true,
      },
      errorsFrame = {
        targetTooClose = true,
        outOfRange = true,
        targetNeedsToBeInFrontOfYou = true,
        thereIsNothingToAttack = true,
        youAreFacingTheWrongWay = true,
        youCantDoThatYet = true,
        youHaveNoTarget = true,
      }
    }
  }
end

function BlizzardUIModule:setDatabase()
  --BlizzardUIModule:Print('BlizzardUIModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('BlizzardUI', self.defaults)
end

function BlizzardUIModule:setOptions()
  --BlizzardUIModule:Print('BlizzardUIModule:setOptions')

  self.options = {
    name = 'BlizzardUI',
    handler = self,
    type = 'group',
    args = {
      buffFrame = {
        order = 1,
        name = 'Buffs',
        type = 'group',
        args = {
          hide = {
            order = 1,
            name = 'Hide',
            type = 'toggle',
            get = function()
              return self.db.profile.buffFrame.hide
            end,
            set = function(_, value)
              self.db.profile.buffFrame.hide = value
              self.buffFrame:checkEnabled()
            end
          },
        }
      },
      castingBarFrame = {
        order = 1,
        name = 'Casting Bar',
        type = 'group',
        args = {
          hide = {
            order = 1,
            name = 'Hide',
            type = 'toggle',
            get = function()
              return self.db.profile.castingBarFrame.hide
            end,
            set = function(_, value)
              self.db.profile.castingBarFrame.hide = value
              self.castingBarFrame:checkEnabled()
            end
          },
        }
      },
      errorsFrame = {
        order = 1,
        name = 'Errors',
        type = 'group',
        args = {
          outOfRange = {
            order = 1,
            name = 'Out of range.',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.outOfRange
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.outOfRange = value
              self.errorsFrame:checkEnabled()
            end
          },

          targetNeedsToBeInFrontOfYou = {
            order = 2,
            name = 'Target needs to be in front of you.',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.targetNeedsToBeInFrontOfYou
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.targetNeedsToBeInFrontOfYou = value
            end
          },

          targetTooClose = {
            order = 3,
            name = 'Target too close',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.targetTooClose
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.targetTooClose = value
            end
          },

          thereIsNothingToAttack = {
            order = 4,
            name = 'There is nothing to attack.',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.thereIsNothingToAttack
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.thereIsNothingToAttack = value
            end
          },

          youAreFacingTheWrongWay = {
            order = 5,
            name = 'You are facing the wrong way!',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.youAreFacingTheWrongWay
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.youAreFacingTheWrongWay = value
            end
          },

          youCantDoThatYet = {
            order = 6,
            name = 'You can\'t do that yet',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.youCantDoThatYet
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.youCantDoThatYet = value
            end
          },

          youHaveNoTarget = {
            order = 7,
            name = 'You have no target',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.youHaveNoTarget
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.youHaveNoTarget = value
            end
          },
        }
      }
    }
  }
end

function BlizzardUIModule:registerOptions()
  --BlizzardUIModule:Print('BlizzardUIModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('idTweaks-BlizzardUI', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('idTweaks-BlizzardUI', 'BlizzardUI', 'idTweaks')
end

function BlizzardUIModule:OnInitialize()
  --BlizzardUIModule:Print('BlizzardUIModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function BlizzardUIModule:OnEnable()
  --BlizzardUIModule:Print('BlizzardUIModule:OnEnable')

  self:registerOptions()

  self.buffFrame:checkEnabled()
  self.castingBarFrame:checkEnabled()
  self.errorsFrame:checkEnabled()
end

function BlizzardUIModule:OnDisable()
  --BlizzardUIModule:Print('BlizzardUIModule:OnDisable')

  self.buffFrame:disable()
  self.castinBarFrame:disable()
  self.errorsFrame:disable()
end

BlizzardUIModule.buffFrame = {}

function BlizzardUIModule.buffFrame:checkEnabled()
  -- BlizzardUIModule:Print('BlizzardUIModule.buffFrame:checkEnabled')

  if BlizzardUIModule.db.profile.buffFrame.hide then
    BlizzardUIModule.buffFrame:enable()
  else
    BlizzardUIModule.buffFrame:disable()
  end
end

function BlizzardUIModule.buffFrame:enable()
  -- BlizzardUIModule:Print('BlizzardUIModule.buffFrame:enable')

  BlizzardUIModule:SecureHook(BuffFrame, 'Show', function(frame)
    if (BlizzardUIModule.db.profile.buffFrame.hide) then
      BuffFrame:Hide()
    end
  end)
  if (BlizzardUIModule.db.profile.buffFrame.hide) then
    BuffFrame:Hide()
  end
end

function BlizzardUIModule.buffFrame:disable()
  -- BlizzardUIModule:Print('BlizzardUIModule.buffFrame:disable')

  BlizzardUIModule:Unhook(BuffFrame, 'Show')

  BuffFrame:Show()
end

BlizzardUIModule.castingBarFrame = {}

function BlizzardUIModule.castingBarFrame:checkEnabled()
  -- BlizzardUIModule:Print('BlizzardUIModule.castingBarFrame:checkEnabled')

  if BlizzardUIModule.db.profile.castingBarFrame.hide then
    BlizzardUIModule.castingBarFrame:enable()
  else
    BlizzardUIModule.castingBarFrame:disable()
  end
end

function BlizzardUIModule.castingBarFrame:enable()
  -- BlizzardUIModule:Print('BlizzardUIModule.castingBarFrame:enable')

  BlizzardUIModule:SecureHook(CastingBarFrame, 'Show', function(frame)
    if (BlizzardUIModule.db.profile.castingBarFrame.hide) then
      CastingBarFrame:Hide()
    end
  end)
end

function BlizzardUIModule.castingBarFrame:disable()
  -- BlizzardUIModule:Print('BlizzardUIModule.castingBarFrame:disable')

  BlizzardUIModule:Unhook(CastingBarFrame, 'Show')
end

BlizzardUIModule.errorsFrame = {}

function BlizzardUIModule.errorsFrame:checkEnabled()
  -- BlizzardUIModule:Print('BlizzardUIModule.errorsFrame:checkEnabled')

  if BlizzardUIModule.db.profile.errorsFrame.hide then
    BlizzardUIModule.errorsFrame:enable()
  else
    BlizzardUIModule.errorsFrame:disable()
  end
end

function BlizzardUIModule.errorsFrame:enable()
  -- BlizzardUIModule:Print('BlizzardUIModule.errorsFrame:enable')

  BlizzardUIModule:RawHook(UIErrorsFrame, 'AddMessage', function(frame, message, red, green, blue, messageId, holdTime)
    for k, v in pairs(BlizzardUIModule.db.profile) do
      if v and message == BlizzardUIModule.options.args[k].name then
        return
      end
    end

    BlizzardUIModule.hooks[frame].AddMessage(frame, message, red, green, blue, messageId, holdTime)
  end, true)
end

function BlizzardUIModule.errorsFrame:disable()
  -- BlizzardUIModule:Print('BlizzardUIModule.errorsFrame:disable')

  BlizzardUIModule:Unhook(UIErrorsFrame, 'AddMessage')
end

idTweaks.BlizzardUI = BlizzardUIModule
