local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')

local MinimapModule = idTweaks:NewModule('Minimap', 'AceConsole-3.0')

function MinimapModule:setDefaults()
  --MinimapModule:Print('MinimapModule:setDefaults')

  self.defaults = {
    profile = {
      zoom = {
        mouseWheel = true
      },
      hide = {
        calendarButton = true,
        timeDisplay = true,
        areaText = true,
        worldMapButton = true,
        zoomButtons = true
      }
    }
  }
end

function MinimapModule:setDatabase()
  --MinimapModule:Print('MinimapModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('Minimap', self.defaults)
end

function MinimapModule:setOptions()
  --MinimapModule:Print('MinimapModule:setOptions')

  self.options = {
    name = 'Minimap',
    desc = 'Changes how the Minimap works.',
    handler = self,
    type = 'group',
    args = {
      zoom = {
        order = 2,
        name = 'Zoom',
        desc = 'Change how zooming the Minimap works.',
        handler = self,
        type = 'group',
        args = {
          mouseWheel = {
            order = 1,
            name = 'Mouse Wheel',
            desc = 'Enables Zooming the minimap with the Mouse Wheel.',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.zoom.mouseWheel
            end,
            set = function(_, value)
              self.db.profile.zoom.mouseWheel = value
              self.zoom:checkMousewheelZooming()
            end
          }
        }
      },

      hide = {
        order = 3,
        name = 'Hide',
        desc = 'Toggle visibility of parts of the Minimap with these options.',
        handler = self,
        type = 'group',
        args = {
          calendarButton = {
            name = 'Calendar Button',
            handler = self,
            type = 'toggle',
            get = function()
              return not self.db.profile.hide.calendarButton
            end,
            set = function(_, value)
              self.db.profile.hide.calendarButton = not value
              self.hide:checkFrame('calendarButton')
            end
          },

          timeDisplay = {
            name = 'Time Display',
            handler = self,
            type = 'toggle',
            get = function()
              return not self.db.profile.hide.timeDisplay
            end,
            set = function(_, value)
              self.db.profile.hide.timeDisplay = not value
              self.hide:checkFrame('timeDisplay')
            end
          },

          areaText = {
            name = 'Area Text',
            handler = self,
            type = 'toggle',
            get = function()
              return not self.db.profile.hide.areaText
            end,
            set = function(_, value)
              self.db.profile.hide.areaText = not value
              self.hide:checkFrame('areaText')
            end
          },

          worldMapButton = {
            name = 'World Map Button',
            handler = self,
            type = 'toggle',
            get = function()
              return not self.db.profile.hide.worldMapButton
            end,
            set = function(_, value)
              self.db.profile.hide.worldMapButton = not value
              self.hide:checkFrame('worldMapButton')
            end
          },

          zoomButtons = {
            name = 'Zoom Buttons',
            handler = self,
            type = 'toggle',
            get = function()
              return not self.db.profile.hide.zoomButtons
            end,
            set = function(_, value)
              self.db.profile.hide.zoomButtons = not value
              self.hide:checkFrame('zoomButtons')
            end
          },
        }
      }
    }
  }
end

function MinimapModule:registerOptions()
  --MinimapModule:Print('MinimapModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('idTweaks-Minimap', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('idTweaks-Minimap', 'Minimap', 'idTweaks')
end

function MinimapModule:OnInitialize()
  --MinimapModule:Print('MinimapModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function MinimapModule:OnEnable()
  --MinimapModule:Print('MinimapModule:OnEnable')

  self:registerOptions()

  self.zoom:checkEnabled()
  self.hide:checkEnabled()
end

function MinimapModule:OnDisable()
  --MinimapModule:Print('MinimapModule:OnDisable')

  self.zoom:disable()
  self.hide:disable()
end

MinimapModule.zoom = {}

function MinimapModule.zoom:disable()
  --MinimapModule:Print('MinimapModule.zoom:disable')

  self:disableMouseWheelZooming()
end

function MinimapModule.zoom:checkEnabled()
  self:checkMousewheelZoomingEnabled()
end

function MinimapModule.zoom:checkMousewheelZoomingEnabled()
  --MinimapModule:Print('MinimapModule.zoom:checkMousewheelZooming')

  if MinimapModule.db.profile.zoom.mouseWheel == true then
    self:enableMouseWheelZooming()
  else
    self:disableMouseWheelZooming()
  end
end

function MinimapModule.zoom:enableMouseWheelZooming()
  Minimap:EnableMouseWheel(true)
  Minimap:SetScript('OnMouseWheel', self.handleMousewheelZoom)
end

function MinimapModule.zoom:disableMouseWheelZooming()
  Minimap:EnableMouseWheel(false)
  Minimap:SetScript('OnMouseWheel', nil)
end

function MinimapModule.zoom.handleMousewheelZoom(frame, delta)
  --MinimapModule:Print('MinimapModule.zoom.handleMousewheelZoom')

  if delta > 0 and Minimap:GetZoom() < 5 then
    Minimap:SetZoom(Minimap:GetZoom() + 1)
  elseif delta < 0 and Minimap:GetZoom() > 0 then
    Minimap:SetZoom(Minimap:GetZoom() - 1)
  end
end

MinimapModule.hide = {
  frames = {
    calendarButton = {
      enable = function()
        GameTimeFrame:Show()
      end,
      disable = function()
        GameTimeFrame:Hide()
      end
    },
    timeDisplay = {
      enable = function()
        TimeManagerClockButton:Show()
      end,
      disable = function()
        TimeManagerClockButton:Hide()
      end
    },
    areaText = {
      enable = function()
        MinimapBorderTop:Show()
        MinimapZoneText:Show()
        MinimapZoneTextButton:Show()
      end,
      disable = function()
        MinimapBorderTop:Hide()
        MinimapZoneText:Hide()
        MinimapZoneTextButton:Hide()
      end
    },
    worldMapButton = {
      enable = function()
        MiniMapWorldMapButton:Show()
      end,
      disable = function()
        MiniMapWorldMapButton:Hide()
      end
    },
    zoomButtons = {
      enable = function()
        MinimapZoomIn:Show()
        MinimapZoomOut:Show()
      end,
      disable = function()
        MinimapZoomIn:Hide()
        MinimapZoomOut:Hide()
      end
    }
  }
}

function MinimapModule.hide:disable()
  --MinimapModule:Print('MinimapModule.hide:disable')

  self:disableAllFrames()
end

function MinimapModule.hide:disableAllFrames()
  for name, functions in pairs(self.frames) do
    self.frames[name].disable()
  end
end

function MinimapModule.hide:checkEnabled()
  --MinimapModule:Print('MinimapModule.hide:checkEnabled')

  for name, functions in pairs(self.frames) do
    self:checkFrame(name)
  end
end

function MinimapModule.hide:checkFrame(name)
  --MinimapModule:Print('MinimapModule.hide:checkFrame')

  if MinimapModule.db.profile.hide[name] == true then
    self.frames[name].enable()
  else
    self.frames[name].disable()
  end
end

idTweaks.Minimap = MinimapModule
