local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')

local TooltipsModule = idTweaks:NewModule('Tooltips', 'AceConsole-3.0', 'AceHook-3.0')

function TooltipsModule:setDefaults()
  --TooltipsModule:Print('TooltipsModule:setDefaults')

  self.defaults = {
    profile = {
      anchor = {
        value = 'ANCHOR_CURSOR'
      }
    }
  }
end

function TooltipsModule:setDatabase()
  --TooltipsModule:Print('TooltipsModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('Tooltips', self.defaults)
end

function TooltipsModule:setOptions()
  --TooltipsModule:Print('TooltipsModule:setOptions')

  self.options = {
    name = 'Tooltips',
    handler = self,
    type = 'group',
    args = {
      anchor = {
        order = 1,
        name = 'Anchor',
        type = 'select',
        values = {
          ['ANCHOR_NONE'] = 'Default',
          ['ANCHOR_CURSOR'] = 'Cursor',
          ['ANCHOR_TOPLEFT'] = 'Top Left',
          ['ANCHOR_TOPRIGHT'] = 'Top Right',
          ['ANCHOR_BOTTOMLEFT'] = 'Bottom Left',
          ['ANCHOR_BOTTOMRIGHT'] = 'Bottom Right',
        },
        get = function()
          return self.db.profile.anchor.value
        end,
        set = function(_, value)
          self.db.profile.anchor.value = value
        end

      }
    }
  }
end

function TooltipsModule:registerOptions()
  --TooltipsModule:Print('TooltipsModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('idTweaks-Tooltips', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('idTweaks-Tooltips', 'Tooltips', 'idTweaks')
end

function TooltipsModule:OnInitialize()
  --TooltipsModule:Print('TooltipsModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function TooltipsModule:OnEnable()
  --TooltipsModule:Print('TooltipsModule:OnEnable')

  self:registerOptions()

  self:SecureHook('GameTooltip_SetDefaultAnchor')
end

function TooltipsModule:OnDisable()
  --TooltipsModule:Print('TooltipsModule:OnDisable')

  self:Unhook('GameTooltip_SetDefaultAnchor')
end

function TooltipsModule:GameTooltip_SetDefaultAnchor(tooltip, selfObject)
  --TooltipsModule:Print('TooltipsModule:GameTooltip_SetDefaultAnchor')

  tooltip:SetOwner(selfObject, TooltipsModule.db.profile.anchor.value)
end

idTweaks.Tooltips = TooltipsModule
